@@ -1,30 +1,57 @@
-/*
+/* -*- Mode: C; tab-width: 4 -*-
+ *
  * Copyright (c) 2002-2003 Apple Computer, Inc. All rights reserved.
  *
- * @APPLE_LICENSE_HEADER_START@
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ *     http://www.apache.org/licenses/LICENSE-2.0
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
- * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
- * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
  * limitations under the License.
- * 
- * @APPLE_LICENSE_HEADER_END@
 
     Change History (most recent first):
 
 $Log: DNSDigest.c,v $
+Revision 1.15.2.1  2006/08/29 06:24:22  cheshire
+Re-licensed mDNSResponder daemon source code under Apache License, Version 2.0
+
+Revision 1.15  2006/06/20 04:12:30  cheshire
+<rdar://problem/4490961> DNS Update broken
+
+Revision 1.14  2006/02/25 23:12:07  cheshire
+<rdar://problem/4427969> Fix to avoid code generation warning/error on FreeBSD 7
+
+Revision 1.13  2004/12/16 20:12:59  cheshire
+<rdar://problem/3324626> Cache memory management improvements
+
+Revision 1.12  2004/12/03 07:20:50  ksekar
+<rdar://problem/3674208> Wide-Area: Registration of large TXT record fails
+
+Revision 1.11  2004/12/02 01:10:27  cheshire
+Fix to compile cleanly on 64-bit x86
+
+Revision 1.10  2004/11/01 20:36:04  ksekar
+<rdar://problem/3802395> mDNSResponder should not receive Keychain Notifications
+
+Revision 1.9  2004/10/26 09:00:12  cheshire
+Save a few bytes by creating HMAC_MD5_AlgName as a C string instead of a 256-byte object
+
+Revision 1.8  2004/09/17 01:08:48  cheshire
+Renamed mDNSClientAPI.h to mDNSEmbeddedAPI.h
+  The name "mDNSClientAPI.h" is misleading to new developers looking at this code. The interfaces
+  declared in that file are ONLY appropriate to single-address-space embedded applications.
+  For clients on general-purpose computers, the interfaces defined in dns_sd.h should be used.
+
+Revision 1.7  2004/08/15 18:36:38  cheshire
+Don't use strcpy() and strlen() on "struct domainname" objects;
+use AssignDomainName() and DomainNameLength() instead
+(A "struct domainname" is a collection of packed pascal strings, not a C string.)
+
 Revision 1.6  2004/06/02 00:17:46  ksekar
 Referenced original OpenSSL license headers in source file description.
 
@@ -53,7 +80,7 @@ Support for TSIG signed dynamic updates.
 extern "C" {
 #endif
 
-#include "mDNSClientAPI.h"
+#include "mDNSEmbeddedAPI.h"
 #include "DNSCommon.h"
 
 // Disable certain benign warnings with Microsoft compilers
@@ -661,7 +688,7 @@ void md5_block_data_order (MD5_CTX *c, const void *p,int num);
  * Time for some action:-)
  */
 
-int HASH_UPDATE (HASH_CTX *c, const void *data_, mDNSu32 len)
+int HASH_UPDATE (HASH_CTX *c, const void *data_, unsigned long len)
 	{
 	const unsigned char *data=(const unsigned char *)data_;
 	register HASH_LONG * p;
@@ -1297,13 +1324,10 @@ mDNSexport mDNSs32 DNSDigest_Base64ToBin(const char *src, mDNSu8 *target, mDNSu3
 #define HMAC_OPAD   0x5c
 #define MD5_LEN     16
 
-static domainname HMAC_MD5_AlgName = { { '\010', 'h', 'm', 'a', 'c', '-', 'm', 'd', '5',
-										 '\007', 's', 'i', 'g', '-', 'a', 'l', 'g',
-										 '\003', 'r', 'e', 'g',
-										 '\003', 'i', 'n', 't',
-										 '\0' } };
+#define HMAC_MD5_AlgName (*(const domainname*) "\010" "hmac-md5" "\007" "sig-alg" "\003" "reg" "\003" "int")
+
 // Adapted from Appendix, RFC 2104
-mDNSexport void DNSDigest_ConstructHMACKey(uDNS_AuthInfo *info, mDNSu8 *key, mDNSu32 len)		
+mDNSexport void DNSDigest_ConstructHMACKey(uDNS_AuthInfo *info, const mDNSu8 *key, mDNSu32 len)		
 	{
 	MD5_CTX k;
 	mDNSu8 buf[MD5_LEN];
@@ -1354,11 +1378,10 @@ mDNSexport mDNSu8 *DNSDigest_SignMessage(DNSMessage *msg, mDNSu8 **end, mDNSu16
 	// Construct TSIG RR, digesting variables as apporpriate
 	mDNSPlatformMemZero(&tsig, sizeof(AuthRecord));	
 	mDNS_SetupResourceRecord(&tsig, mDNSNULL, 0, kDNSType_TSIG, 0, kDNSRecordTypeKnownUnique, mDNSNULL, mDNSNULL);
-	rdata = tsig.resrec.rdata->u.data;
 
 	// key name
-	mDNSPlatformStrCopy(info->keyname.c, tsig.resrec.name.c);
-	MD5_Update(&c, info->keyname.c, mDNSPlatformStrLen(info->keyname.c)+1);
+	AssignDomainName(tsig.resrec.name, &info->keyname);
+	MD5_Update(&c, info->keyname.c, DomainNameLength(&info->keyname));
 
 	// class
 	tsig.resrec.rrclass = kDNSQClass_ANY;
@@ -1370,9 +1393,9 @@ mDNSexport mDNSu8 *DNSDigest_SignMessage(DNSMessage *msg, mDNSu8 **end, mDNSu16
 	MD5_Update(&c, (mDNSu8 *)&tsig.resrec.rroriginalttl, sizeof(tsig.resrec.rroriginalttl));
 	
 	// alg name
-	mDNSPlatformStrCopy(HMAC_MD5_AlgName.c, rdata);     
-	len = mDNSPlatformStrLen(HMAC_MD5_AlgName.c) + 1;
-	rdata += len;
+	AssignDomainName(&tsig.resrec.rdata->u.name, &HMAC_MD5_AlgName);
+	len = DomainNameLength(&HMAC_MD5_AlgName);
+	rdata = tsig.resrec.rdata->u.data + len;
 	MD5_Update(&c, HMAC_MD5_AlgName.c, len);
 
 	// time
@@ -1390,11 +1413,11 @@ mDNSexport mDNSu8 *DNSDigest_SignMessage(DNSMessage *msg, mDNSu8 **end, mDNSu16
 	rdata += 6;              	
 	MD5_Update(&c, utc48, 6);
 
-	// fudge
-	buf = mDNSOpaque16fromIntVal(300);     // 300 sec is fudge recommended in RFC 2485
-	((mDNSOpaque16 *)rdata)->NotAnInteger = buf.NotAnInteger;
+	// 300 sec is fudge recommended in RFC 2485
+	rdata[0] = (mDNSu8)((300 >> 8)  & 0xff);
+	rdata[1] = (mDNSu8)( 300        & 0xff);
+	MD5_Update(&c, rdata, sizeof(mDNSOpaque16));
 	rdata += sizeof(mDNSOpaque16);
-	MD5_Update(&c, buf.b, sizeof(mDNSOpaque16));
 
 	// digest error and other data len (both zero) - we'll add them to the rdata later
 	buf.NotAnInteger = 0;
@@ -1411,19 +1434,21 @@ mDNSexport mDNSu8 *DNSDigest_SignMessage(DNSMessage *msg, mDNSu8 **end, mDNSu16
 	MD5_Final(digest, &c);
 
 	// set remaining rdata fields
-	*(mDNSOpaque16 *)rdata = mDNSOpaque16fromIntVal(MD5_LEN);             // MAC size	
+	rdata[0] = (mDNSu8)((MD5_LEN >> 8)  & 0xff);
+	rdata[1] = (mDNSu8)( MD5_LEN        & 0xff);
 	rdata += sizeof(mDNSOpaque16);
 	mDNSPlatformMemCopy(digest, rdata, MD5_LEN);                          // MAC
 	rdata += MD5_LEN;
-	((mDNSOpaque16 *)rdata)->NotAnInteger = msg->h.id.NotAnInteger;       // original ID
-	rdata += sizeof(mDNSOpaque16);
-	((mDNSOpaque16 *)rdata)->NotAnInteger = 0;                            // no error
-	rdata += sizeof(mDNSOpaque16);
-	((mDNSOpaque16 *)rdata)->NotAnInteger = 0;                            // other data len
-	rdata += sizeof(mDNSOpaque16);
+	rdata[0] = msg->h.id.b[0];                                            // original ID
+	rdata[1] = msg->h.id.b[1];
+	rdata[2] = 0;                                                         // no error
+	rdata[3] = 0;
+	rdata[4] = 0;                                                         // other data len
+	rdata[5] = 0;
+	rdata += 6;
 	
 	tsig.resrec.rdlength = (mDNSu16)(rdata - tsig.resrec.rdata->u.data);
-	*end = PutResourceRecordTTL(msg, ptr, numAdditionals, &tsig.resrec, 0);
+	*end = PutResourceRecordTTLJumbo(msg, ptr, numAdditionals, &tsig.resrec, 0);
 	if (!*end) { LogMsg("ERROR: DNSDigest_SignMessage - could not put TSIG"); return mDNSNULL; }
 
 	// update num additionals
@@ -1433,7 +1458,6 @@ mDNSexport mDNSu8 *DNSDigest_SignMessage(DNSMessage *msg, mDNSu8 **end, mDNSu16
 
 	return *end;
 	}
-	
 
 #ifdef __cplusplus
 }
