@@ -2,33 +2,23 @@
  *
  * Copyright (c) 2004 Apple Computer, Inc. All rights reserved.
  *
- * @APPLE_LICENSE_HEADER_START@
- *
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- *
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
- * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
- * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * @APPLE_LICENSE_HEADER_END@
 
     Change History (most recent first):
 
 $Log: LegacyNATTraversal.c,v $
-Revision 1.14  2005/12/08 03:00:33  cheshire
-<rdar://problem/4349971> Byte order bugs in Legacy NAT traversal code
-
-Revision 1.13  2005/09/07 18:23:05  ksekar
-<rdar://problem/4151514> Off-by-one overflow in LegacyNATTraversal
+Revision 1.12.2.1  2006/08/29 06:24:30  cheshire
+Re-licensed mDNSResponder daemon source code under Apache License, Version 2.0
 
 Revision 1.12  2005/07/22 21:36:16  ksekar
 Fix GCC 4.0/Intel compiler warnings
@@ -112,8 +102,6 @@ Revision 1.1  2004/08/18 17:35:41  ksekar
 // TODO: remove later and do variable length
 #define MAX_SOAPMSGSIZE		65536
 
-// This code accidentally closes fd 0 all over the place
-// To stop that messing up the mDNSResponder core, we trap it and prevent it
 static int safe_close(int fd)
 	{
 	if (fd < 3) { /* LogMsg("safe_close: ERROR sd %d < 3", fd); */ return(-1); }
@@ -122,10 +110,6 @@ static int safe_close(int fd)
 
 #define close safe_close
 
-// This code uses fprintf(stderr, ...) and similar to log error messages
-// We redirect all of them to syslog using our LogMsg mechanism
-#define fprintf(file, ...) LogMsg(__VA_ARGS__)
-
 ////////////////////////////////////////////////////////////////////////
 // NetAddr Functions
 ////////////////////////////////////////////////////////////////////////
@@ -448,7 +432,7 @@ typedef struct tagIPINFO
 {
         int                             iFlags;
         char                    szIfName[IFNAMELEN];    /* Interface name                       */
-        unsigned char   abIP[IPLEN];
+        unsigned char   abIP[IPLEN];                    /* IP in host byte order        */
         unsigned short  wPort;
 } IPINFO, *PIPINFO, **PPIPINFO;
 
@@ -456,13 +440,10 @@ typedef struct hostent	HOSTENT, *PHOSTENT;
 
 static unsigned long GetNATIPNetmask(unsigned long dwIP)
 {
-	static const union { uint8_t b[4]; uint32_t l; } mask_10 = { { 255, 0,   0, 0 } };	// Mask for 10/8
-	static const union { uint8_t b[4]; uint32_t l; } mask172 = { { 255, 240, 0, 0 } };	// Mask for 172.16/12
-	static const union { uint8_t b[4]; uint32_t l; } mask192 = { { 255, 255, 0, 0 } };	// Mask for 192.168/16
-	uint8_t *p = (uint8_t *)&dwIP;
-	if (p[0] ==  10                       ) return mask_10.l;
-	if (p[0] == 172 && (p[1] & 0xF0) == 16) return mask172.l;
-	if (p[0] == 192 && p[1] == 168        ) return mask192.l;
+	if ((dwIP & 0xFF000000) == 0x0A000000)  return 0xFF000000;
+	if ((dwIP & 0xFFF00000) == 0xAC100000)  return 0xFFF00000;
+	if ((dwIP & 0xFFFF0000) == 0xC0a80000)  return 0xFFFF0000;
+
 	return 0;	/* No NAT IP */
 }
 
@@ -546,7 +527,8 @@ static int GetIPInfo(PPIPINFO ppIPInfo)
 		{
 		case AF_INET:
 			memcpy(pIPInfo[iNum].szIfName, ifr->ifr_name, IFNAMELEN);
-			dwIP = ((struct sockaddr_in *)&ifr->ifr_addr)->sin_addr.s_addr;
+			dwIP =
+				ntohl(((struct sockaddr_in *)&ifr->ifr_addr)->sin_addr.s_addr);
 			memcpy(pIPInfo[iNum].abIP, &dwIP, sizeof(unsigned long));
 			if (ifrcopy.ifr_flags & IFF_POINTOPOINT)
 				pIPInfo[iNum].iFlags |= ISPPP;
@@ -619,7 +601,7 @@ static int SSDPListen()
 	saddr.sin_family = AF_INET;
 	//saddr.sin_addr.s_addr = inet_addr(SSDP_IP);
 	//saddr.sin_port = htons(SSDP_PORT);
-	saddr.sin_addr.s_addr = g_dwLocalIP;
+	saddr.sin_addr.s_addr = htonl(g_dwLocalIP);
 	saddr.sin_port = 0;
 
 	// and set the multicast add_member structure
@@ -664,7 +646,7 @@ static int EventListen()
 		bzero(&saddr, sizeof(saddr));
 		saddr.sin_len = sizeof(saddr);
 		saddr.sin_family = AF_INET;
-		saddr.sin_addr.s_addr = g_dwLocalIP;
+		saddr.sin_addr.s_addr = htonl(g_dwLocalIP);
 		saddr.sin_port = htons(g_wEventPort);
 
 		// return if okay
@@ -735,7 +717,7 @@ static void DumpHex(char *buf, int len)
 			nexti = i + 16;
 			endj = (nexti > len) ? len : nexti;
 			for (j = i; j < endj; j++)
-				fprintf(g_log, "%02x %c ", buf[j] & 0xff, buf[j]);
+				fprintf(g_log, "%02x ", buf[j] & 0xff);
 			if (j == len) {
 				if ((j % 16) != 0) {
 					char pad[3 * 16 + 1];  // don't need the last 3 bytes anyway
@@ -774,7 +756,7 @@ static char *FindHTTPHeaderNewLine(char *pbuf, int iBufSize, int *pfEOH)
 		result = memchr(pbuf, '\r', iBufSize);
 		if (result == NULL) {
 			if (g_fLogging & NALOG_INFO0) {
-				fprintf(g_log, "FindHTTPHeaderNewLine: er @(%d/%d)\n", i, iBufSize);
+				fprintf(g_log, "FindHTTPHeaderNewLine: er @(%d)\n", i);
 				fflush(g_log);
 			}
 			return NULL;
@@ -912,15 +894,7 @@ static PHTTPResponse NewHTTPResponse_sz(
 		pszEOL = FindHTTPHeaderNewLine(pszEOL,
 			iBufferSize - (pszEOL - pBuf),  // remainder size
 			&fEOH);
-		if (pszEOL == NULL) {
-			if (g_fLogging & NALOG_INFO0) {
-				fprintf(g_log, "NewHTTPResponse_sz: er reading header field %d @ %lu / %lu\n",
-					iNumHeaders, pHeader->pszName - pBuf, iBufferSize);
-				DumpHex(pszHTTPResponse, iBufferSize);
-				fflush(g_log);
-			}
-			goto cleanup;  // syntax error
-		}
+		if (pszEOL == NULL) goto cleanup;  // syntax error
 
 		*pszEOL = '\0';  // terminate this string
 		pszEOL += 2;  // point to beginning of next line
@@ -1330,9 +1304,12 @@ static void *TCPProc(void *in)
 		char response[2000];
 		PHTTPResponse resp;
 		int n;
-		sprintf(callback, "%u.%u.%u.%u:%u",
-			((uint8_t*)&g_dwLocalIP)[0], ((uint8_t*)&g_dwLocalIP)[1],
-			((uint8_t*)&g_dwLocalIP)[2], ((uint8_t*)&g_dwLocalIP)[3], g_wEventPort);
+		sprintf(callback, "%lu.%lu.%lu.%lu:%u", 
+			(g_dwLocalIP >> 24) & 0xFF,
+			(g_dwLocalIP >> 16) & 0xFF,
+			(g_dwLocalIP >> 8) & 0xFF,
+			(g_dwLocalIP >> 0) & 0xFF,
+			g_wEventPort);
 
 		n = sprintf((char *)buf,
 			szEventMsgSubscribeFMT,
@@ -1347,7 +1324,7 @@ static void *TCPProc(void *in)
 		if (n > 0)
 		{
 			response[n] = '\0';
-			resp = NewHTTPResponse_sz((char *)response, n+1, TRUE);
+			resp = NewHTTPResponse_sz((char *)buf, n, TRUE);
 			if (NULL != resp)
 			{
 ////TracePrint(ELL_TRACE, "UPnP Subscribe returns %s/%d\n", resp->pszStatus, n);
@@ -1523,7 +1500,7 @@ static void *UDPProc(void *in)
 
 		if (!FD_ISSET(g_sUDP, &readfds)) continue;
 		recvaddrlen = sizeof(recvaddr);
-		n = recvfrom(g_sUDP, buf, sizeof(buf)-1, 0,
+		n = recvfrom(g_sUDP, buf, sizeof(buf), 0,
 			(struct sockaddr *)&recvaddr, &recvaddrlen);
 		if (n < 0) {
 			if (g_fLogging & NALOG_ERROR)
@@ -1536,7 +1513,7 @@ static void *UDPProc(void *in)
 		}
 		buf[n] = '\0';
 		if (strncmp((char *)buf, "HTTP/1.1", 8) == 0) {
-			PHTTPResponse pResponse = NewHTTPResponse_sz((char *)buf, n+1, TRUE);
+			PHTTPResponse pResponse = NewHTTPResponse_sz((char *)buf, n, TRUE);
 			PrintHTTPResponse(pResponse);
 			if (DiscoverRouter(pResponse) == 0)
 			{
@@ -1555,7 +1532,7 @@ static void *UDPProc(void *in)
 			// temporarily use this to fudge - will have the exact same
 			// parsing, only status/reason set to "*" and "HTTP/1.1".
 			// TODO: add support for HTTP requests
-			PHTTPResponse pResponse = NewHTTPResponse_sz((char *)buf, n+1, TRUE);
+			PHTTPResponse pResponse = NewHTTPResponse_sz((char *)buf, n, TRUE);
 			if (DiscoverRouter(pResponse) == 0)
 			{
 				time_t	now = time(NULL);
@@ -1877,14 +1854,13 @@ GetTimeElapsed(&tv_start, &tv_end, &tv_elapsed);
 
 		FD_ZERO(&readfds);
 		FD_SET(s, &readfds);
-
-		// In testing, the Linksys Wireless-G Broadband Router "WRT54GS" takes
-		// up to four seconds to respond, and even then only a partial response,
-		// with the remainder coming in a second TCP segment half a second later.
-		// Accordingly, we wait up to five seconds for the initial data, and then after that
-		// wait one second after subsequent TCP segments, in care more data is still coming.
-		timeout.tv_sec  = iBufLen ? 1 : 5;
+		//timeout.tv_sec = g_iFunctionTimeout / U_TOGRAN;
+		//timeout.tv_usec = (g_iFunctionTimeout % U_TOGRAN) * 1000000 / U_TOGRAN;
+		// just do flat 2 sec now, since connection already established
+		timeout.tv_sec = 1;
 		timeout.tv_usec = 0;
+
+
 		iRet = select(s+1, &readfds, NULL, NULL, &timeout);
 		if (iRet <= 0)
 		{
@@ -1949,7 +1925,7 @@ GetTimeElapsed(&tv_start, &tv_end, &tv_elapsed);
 //fprintf(stderr, "2 -- \n");
 
 	if (g_fLogging & NALOG_INFO1)
-		fprintf(g_log, "SendTCPMsg_saddr_2part done recv %d @ %lu\n", iBufLen, time(NULL));
+		fprintf(g_log, "done recv @%lu\n", time(NULL));
 
 	if (result == NULL) {    // if caller just want to send/display msgs
 		if (g_fLogging & NALOG_DUMP)
@@ -2038,7 +2014,7 @@ static int SendTCPMsg_saddr_parse(
 	}
 
 	if (g_fLogging & NALOG_INFO1)
-		fprintf(g_log, "SendTCPMsg_saddr_parse: Before Sending TCP Msg: %d == %lu?\n",
+		fprintf(g_log, "SendTCPMsg/parse: Before Sending TCP Msg: %d == %lu?\n",
 			iLen, strlen(msg));
 	if (g_fLogging & NALOG_DUMP)
 		fprintf(g_log,"Sending TCP msg:\n[%s]\n", msg);
@@ -2090,14 +2066,12 @@ static int SendTCPMsg_saddr_parse(
 
 		FD_ZERO(&readfds);
 		FD_SET(s, &readfds);
-
-		// In testing, the Linksys Wireless-G Broadband Router "WRT54GS" takes
-		// up to four seconds to respond, and even then only a partial response,
-		// with the remainder coming in a second TCP segment half a second later.
-		// Accordingly, we wait up to five seconds for the initial data, and then after that
-		// wait one second after subsequent TCP segments, in care more data is still coming.
-		timeout.tv_sec  = iBufLen ? 1 : 5;
+		//timeout.tv_sec = g_iFunctionTimeout / U_TOGRAN;
+		//timeout.tv_usec = (g_iFunctionTimeout % U_TOGRAN) * 1000000 / U_TOGRAN;
+		// just do flat 2 sec now, since connection already established
+		timeout.tv_sec = 1;
 		timeout.tv_usec = 0;
+
 		iRet = select(s+1, &readfds, NULL, NULL, &timeout);
 		if (iRet <= 0) {
 //fprintf(stderr, "**********: select failed (%d/%d)\n", iRet, errno);
@@ -2124,8 +2098,6 @@ static int SendTCPMsg_saddr_parse(
 		if (resultSize <= iBufLen) {
 			char	t[1000];
 			i = recv(s, &t, 1000, 0);
-			if (g_fLogging & NALOG_INFO1)
-				fprintf(g_log, "SendTCPMsg_saddr_parse discarding %d bytes\n", i);
 			if (i== 0) break;
 			// Note that there's no dump here - prevents DoS attack from
 			// flooding the logs/diskspace
@@ -2133,8 +2105,6 @@ static int SendTCPMsg_saddr_parse(
 		}
 
 		i = recv(s, result + iBufLen, resultSize - iBufLen, 0);
-		if (g_fLogging & NALOG_INFO1)
-			fprintf(g_log, "SendTCPMsg_saddr_parse read %d bytes (%d/%d)\n", i, iBufLen, resultSize);
 		if (0 == i) {
 
 			break;
@@ -2157,7 +2127,7 @@ static int SendTCPMsg_saddr_parse(
 //fprintf(stderr, "p -- \n");
 
 	if (g_fLogging & NALOG_INFO1)
-		fprintf(g_log, "SendTCPMsg_saddr_parse done recv %d @ %lu\n", iBufLen, time(NULL));
+		fprintf(g_log, "done recv @%lu\n", time(NULL));
 
 	if (result == NULL) {    // if caller just want to send/display msgs
 		if (g_fLogging & NALOG_DUMP)
@@ -2269,8 +2239,6 @@ static PHTTPResponse SendSOAPMsgControlAction(
 			&g_saddrRouterSOAP);
 	}
 
-	if (g_fLogging & NALOG_INFO1)
-		fprintf(g_log, "SendSOAPMsgControlAction iResultLen %d\n", iResultLen);
 	if (iResultLen > 0) {
 		if (iResultLen > MAX_SOAPMSGSIZE) {
 			if (g_fLogging & NALOG_ALERT)
@@ -2501,8 +2469,10 @@ static void ParseURL(
 	szBuf,
 	pszHostPort?pszHostPort:"",
 	pszPath?pszPath:"",
-	((uint8_t*)&psaddr->sin_addr.s_addr)[0], ((uint8_t*)&psaddr->sin_addr.s_addr)[1],
-	((uint8_t*)&psaddr->sin_addr.s_addr)[2], ((uint8_t*)&psaddr->sin_addr.s_addr)[3],
+	(psaddr->sin_addr.s_addr >> 24) & 0xff,
+	(psaddr->sin_addr.s_addr >> 16) & 0xff,
+	(psaddr->sin_addr.s_addr >> 8) & 0xff,
+	(psaddr->sin_addr.s_addr >> 0) & 0xff,
 	psaddr->sin_port);
 #endif
 }
@@ -2632,13 +2602,17 @@ static void GetIPByName(char *hostname, unsigned long *ip_ret)
 		if (pHEnt == NULL) {
 			if (g_fLogging & NALOG_ALERT)
 				fprintf(g_log, "Can't translate [%s] to IP...\n", hostname);
-			g_dwLocalIP = INADDR_ANY;
+			g_dwLocalIP = htonl(INADDR_ANY);
 			return;
 		}
-		ip = *(unsigned long *)(pHEnt->h_addr);
+		ip = ntohl(*(unsigned long *)(pHEnt->h_addr));
 		if (g_fLogging & NALOG_INFO1)
-			fprintf(g_log, "hostname [%s] to ip: %u.%u.%u.%u\n", hostname,
-				((uint8_t*)&ip)[0], ((uint8_t*)&ip)[1], ((uint8_t*)&ip)[2], ((uint8_t*)&ip)[3]);
+			fprintf(g_log, "hostname [%s] to ip: %ld.%ld.%ld.%ld\n",
+				hostname,
+				(ip >> 24) & 0xff,
+				(ip >> 16) & 0xff,
+				(ip >> 8) & 0xff,
+				(ip >> 0) & 0xff);
 	}
 	*ip_ret = ip;
 }
@@ -2706,8 +2680,9 @@ mStatus LNT_UnmapPort(mDNSIPPort PubPort, mDNSBool tcp)
 	//unsigned		long dwIP;
 	Property		propArgs[3];
 	PHTTPResponse	resp;
+    unsigned short port = PubPort.NotAnInteger;
     int protocol = tcp ? IPPROTO_TCP : IPPROTO_UDP;
-	sprintf(szEPort, "%u", mDNSVal16(PubPort));
+	sprintf(szEPort, "%u", port);
 
 	bzero(propArgs, sizeof(propArgs));
 	propArgs[0].pszName = "NewRemoteHost";
@@ -2755,19 +2730,26 @@ extern mStatus LNT_MapPort(mDNSIPPort priv, mDNSIPPort pub, mDNSBool tcp)
 	char			descr[40];
 	Property		propArgs[8];
 	PHTTPResponse	resp;
+    unsigned short iport = priv.NotAnInteger;
+    unsigned short eport = pub.NotAnInteger;
     int protocol = tcp ? IPPROTO_TCP : IPPROTO_UDP;
 
-	if (NA_E_EXISTS == GetMappingUnused(mDNSVal16(pub), protocol))
+
+	if (NA_E_EXISTS == GetMappingUnused(eport, protocol))
 		return mStatus_AlreadyRegistered;
 
 	//DeletePortMapping(eport, protocol);
 
-	sprintf(szEPort, "%u", mDNSVal16(pub));
-	sprintf(szIPort, "%u", mDNSVal16(priv));
+	sprintf(szEPort, "%u", eport);
+
+	sprintf(szIPort, "%u", iport);
 
 	dwIP = g_dwLocalIP;
 	sprintf(szLocalIP, "%u.%u.%u.%u",
-		((uint8_t*)&dwIP)[0], ((uint8_t*)&dwIP)[1], ((uint8_t*)&dwIP)[2], ((uint8_t*)&dwIP)[3]);
+		(unsigned int)((dwIP >> 24) & 0xff),
+		(unsigned int)((dwIP >> 16) & 0xff),
+		(unsigned int)((dwIP >> 8) & 0xff),
+		(unsigned int)((dwIP >> 0) & 0xff));
 
 	bzero(propArgs, sizeof(propArgs));
 	propArgs[0].pszName = "NewRemoteHost";
@@ -2797,7 +2779,7 @@ extern mStatus LNT_MapPort(mDNSIPPort priv, mDNSIPPort pub, mDNSBool tcp)
 	propArgs[5].pszValue = "1";
 	propArgs[5].pszType = "boolean";
 	propArgs[6].pszName = "NewPortMappingDescription";
-	sprintf(descr, "iC%u", mDNSVal16(pub));
+	sprintf(descr, "iC%u", eport);
 	//propArgs[6].pszValue = "V";
 	propArgs[6].pszValue = descr;
 	propArgs[6].pszType = "string";
@@ -2805,15 +2787,9 @@ extern mStatus LNT_MapPort(mDNSIPPort priv, mDNSIPPort pub, mDNSBool tcp)
 	propArgs[7].pszValue = "0";
 	propArgs[7].pszType = "ui4";
 
-	if (g_fLogging & NALOG_INFO1)
-		fprintf(g_log, "Sending AddPortMapping priv %u pub %u\n", mDNSVal16(priv), mDNSVal16(pub));
-
 	resp = SendSOAPMsgControlAction(
 		"AddPortMapping", 8, propArgs, FALSE);
 
-	if (g_fLogging & NALOG_INFO1)
-		fprintf(g_log, "AddPortMapping resp %p\n", resp);
-
 	if (resp == NULL) {
 		return mStatus_NATTraversal;
 	}
@@ -2934,13 +2910,11 @@ int LegacyNATInit(void)
 	pthread_attr_t	attr;
 	int				iRet;
 	//struct timeval	tv;
-	LogOperation("LegacyNATInit");
 
 	static int		fFirstInitLocks = TRUE;
 	FILE *log = NULL;	
 	
 	g_fLogging = 0;
-	//g_fLogging = ~0;		// Turns ALL logging on
 	g_log = stderr;
 
 	SetLocalIP();
