@@ -1,26 +1,18 @@
-/*
+/* -*- Mode: C; tab-width: 4 -*-
+ *
  * Copyright (c) 2002-2003 Apple Computer, Inc. All rights reserved.
  *
- * @APPLE_LICENSE_HEADER_START@
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ *     http://www.apache.org/licenses/LICENSE-2.0
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
- * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
- * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
  * limitations under the License.
- * 
- * @APPLE_LICENSE_HEADER_END@
  *
  * Formatting notes:
  * This code follows the "Whitesmiths style" C indentation rules. Plenty of discussion
@@ -38,6 +30,18 @@
     Change History (most recent first):
 
 $Log: SamplemDNSClient.c,v $
+Revision 1.48  2006/08/14 23:24:39  cheshire
+Re-licensed mDNSResponder daemon source code under Apache License, Version 2.0
+
+Revision 1.47  2006/01/10 02:29:22  cheshire
+<rdar://problem/4403861> Cosmetic IPv6 address display problem in mDNS test tool
+
+Revision 1.46  2004/11/02 01:32:34  cheshire
+<rdar://problem/3861705> Update code so it still compiles when DNSServiceDiscovery.h is deprecated
+
+Revision 1.45  2004/06/15 02:39:47  cheshire
+When displaying error message, only show command name, not entire path
+
 Revision 1.44  2004/05/28 02:20:06  cheshire
 If we allow dot or empty string for domain when resolving a service,
 it should be a synonym for "local"
@@ -80,6 +84,12 @@ Add checkin history header
 #include <arpa/inet.h>
 #include <net/if.h>
 #include <CoreFoundation/CoreFoundation.h>
+
+// We already know this tool is using the old deprecated API (that's its purpose)
+// Since we compile with all warnings treated as errors, we have to turn off the warnings here or the project won't compile
+#include <AvailabilityMacros.h>
+#undef AVAILABLE_MAC_OS_X_VERSION_10_2_AND_LATER_BUT_DEPRECATED
+#define AVAILABLE_MAC_OS_X_VERSION_10_2_AND_LATER_BUT_DEPRECATED
 #include <DNSServiceDiscovery/DNSServiceDiscovery.h>
 
 //*************************************************************************************************************
@@ -207,12 +217,14 @@ static void resolve_reply(struct sockaddr *interface, struct sockaddr *address,
         else if (address->sa_family == AF_INET6)
             {
             struct sockaddr_in6 *ip6 = (struct sockaddr_in6 *)address;
-            u_int16_t *w = ip6->sin6_addr.__u6_addr.__u6_addr16;
+            u_int8_t *b = ip6->sin6_addr.__u6_addr.__u6_addr8;
             union { uint16_t s; u_char b[2]; } port = { ip6->sin6_port };
             uint16_t PortAsNumber = ((uint16_t)port.b[0]) << 8 | port.b[1];
             char ipstring[40];
             char ifname[IF_NAMESIZE + 1] = "";
-            sprintf(ipstring, "%04X:%04X:%04X:%04X:%04X:%04X:%04X:%04X", w[0], w[1], w[2], w[3], w[4], w[5], w[6], w[7]);
+            sprintf(ipstring, "%02X%02X:%02X%02X:%02X%02X:%02X%02X:%02X%02X:%02X%02X:%02X%02X:%02X%02X",
+				b[0x0], b[0x1], b[0x2], b[0x3], b[0x4], b[0x5], b[0x6], b[0x7],
+				b[0x8], b[0x9], b[0xA], b[0xB], b[0xC], b[0xD], b[0xE], b[0xF]);
             if (ip6->sin6_scope_id) { ifname[0] = '%';  if_indextoname(ip6->sin6_scope_id, &ifname[1]); }
             printf("%s%s:%u", ipstring, ifname, PortAsNumber);
             }
@@ -322,6 +334,7 @@ static void reg_reply(DNSServiceRegistrationReplyErrorType errorCode, void *cont
 
 int main(int argc, char **argv)
 	{
+	const char *progname = strrchr(argv[0], '/') ? strrchr(argv[0], '/') + 1 : argv[0];
 	char *dom;
 	setlinebuf(stdout);				// Want to see lines as they appear, not block buffered
 
@@ -443,16 +456,16 @@ Exit:
 	return 0;
 
 Fail:
-	fprintf(stderr, "%s -E                  (Enumerate recommended registration domains)\n", argv[0]);
-	fprintf(stderr, "%s -F                      (Enumerate recommended browsing domains)\n", argv[0]);
-	fprintf(stderr, "%s -B        <Type> <Domain>        (Browse for services instances)\n", argv[0]);
-	fprintf(stderr, "%s -L <Name> <Type> <Domain>           (Look up a service instance)\n", argv[0]);
-	fprintf(stderr, "%s -R <Name> <Type> <Domain> <Port> [<TXT>...] (Register a service)\n", argv[0]);
-	fprintf(stderr, "%s -A                      (Test Adding/Updating/Deleting a record)\n", argv[0]);
-	fprintf(stderr, "%s -U                                  (Test updating a TXT record)\n", argv[0]);
-	fprintf(stderr, "%s -N                             (Test adding a large NULL record)\n", argv[0]);
-	fprintf(stderr, "%s -T                            (Test creating a large TXT record)\n", argv[0]);
-	fprintf(stderr, "%s -M      (Test creating a registration with multiple TXT records)\n", argv[0]);
-	fprintf(stderr, "%s -I   (Test registering and then immediately updating TXT record)\n", argv[0]);
+	fprintf(stderr, "%s -E                  (Enumerate recommended registration domains)\n", progname);
+	fprintf(stderr, "%s -F                      (Enumerate recommended browsing domains)\n", progname);
+	fprintf(stderr, "%s -B        <Type> <Domain>        (Browse for services instances)\n", progname);
+	fprintf(stderr, "%s -L <Name> <Type> <Domain>           (Look up a service instance)\n", progname);
+	fprintf(stderr, "%s -R <Name> <Type> <Domain> <Port> [<TXT>...] (Register a service)\n", progname);
+	fprintf(stderr, "%s -A                      (Test Adding/Updating/Deleting a record)\n", progname);
+	fprintf(stderr, "%s -U                                  (Test updating a TXT record)\n", progname);
+	fprintf(stderr, "%s -N                             (Test adding a large NULL record)\n", progname);
+	fprintf(stderr, "%s -T                            (Test creating a large TXT record)\n", progname);
+	fprintf(stderr, "%s -M      (Test creating a registration with multiple TXT records)\n", progname);
+	fprintf(stderr, "%s -I   (Test registering and then immediately updating TXT record)\n", progname);
 	return 0;
 	}
