@@ -1,30 +1,62 @@
-/*
- * Copyright (c) 2002-2003 Apple Computer, Inc. All rights reserved.
+/* -*- Mode: C; tab-width: 4 -*-
  *
- * @APPLE_LICENSE_HEADER_START@
+ * Copyright (c) 2002-2004 Apple Computer, Inc. All rights reserved.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ *     http://www.apache.org/licenses/LICENSE-2.0
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
- * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
- * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
  * limitations under the License.
- * 
- * @APPLE_LICENSE_HEADER_END@
 
     Change History (most recent first):
 
 $Log: Responder.c,v $
+Revision 1.32  2006/08/14 23:24:46  cheshire
+Re-licensed mDNSResponder daemon source code under Apache License, Version 2.0
+
+Revision 1.31  2006/06/12 18:22:42  cheshire
+<rdar://problem/4580067> mDNSResponder building warnings under Red Hat 64-bit (LP64) Linux
+
+Revision 1.30  2005/10/26 22:21:16  cheshire
+<rdar://problem/4149841> Potential buffer overflow in mDNSResponderPosix
+
+Revision 1.29  2005/03/04 21:35:33  cheshire
+<rdar://problem/4037201> Services.txt file not parsed properly when it contains more than one service
+
+Revision 1.28  2005/01/11 01:55:26  ksekar
+Fix compile errors in Posix debug build
+
+Revision 1.27  2004/12/01 04:28:43  cheshire
+<rdar://problem/3872803> Darwin patches for Solaris and Suse
+Use version of daemon() provided in mDNSUNP.c instead of local copy
+
+Revision 1.26  2004/11/30 22:37:01  cheshire
+Update copyright dates and add "Mode: C; tab-width: 4" headers
+
+Revision 1.25  2004/11/11 02:00:51  cheshire
+Minor fixes to getopt, error message
+
+Revision 1.24  2004/11/09 19:32:10  rpantos
+Suggestion from Ademar de Souza Reis Jr. to allow comments in services file
+
+Revision 1.23  2004/09/17 01:08:54  cheshire
+Renamed mDNSClientAPI.h to mDNSEmbeddedAPI.h
+  The name "mDNSClientAPI.h" is misleading to new developers looking at this code. The interfaces
+  declared in that file are ONLY appropriate to single-address-space embedded applications.
+  For clients on general-purpose computers, the interfaces defined in dns_sd.h should be used.
+
+Revision 1.22  2004/09/16 01:58:22  cheshire
+Fix compiler warnings
+
+Revision 1.21  2004/06/15 03:48:07  cheshire
+Update mDNSResponderPosix to take multiple name=val arguments in a sane way
+
 Revision 1.20  2004/05/18 23:51:26  cheshire
 Tidy up all checkin comments to use consistent "<rdar://problem/xxxxxxx>" format for bug numbers
 
@@ -89,7 +121,7 @@ First checkin
 
 */
 
-#include "mDNSClientAPI.h"// Defines the interface to the client layer above
+#include "mDNSEmbeddedAPI.h"// Defines the interface to the client layer above
 #include "mDNSPosix.h"    // Defines the specific types needed to run mDNS on this platform
 
 #include <assert.h>
@@ -184,42 +216,26 @@ static void HandleSigQuit(int sigraised)
 #pragma mark ***** Parameter Checking
 #endif
 
-static mDNSBool CheckThatRichTextHostNameIsUsable(const char *richTextHostName, mDNSBool printExplanation)
-    // Checks that richTextHostName is a reasonable host name 
+static mDNSBool CheckThatRichTextNameIsUsable(const char *richTextName, mDNSBool printExplanation)
+    // Checks that richTextName is reasonable 
     // label and, if it isn't and printExplanation is true, prints 
     // an explanation of why not.
 {
-    mDNSBool    result;
-    domainlabel richLabel;
-    domainlabel poorLabel;
-    
-    result = mDNStrue;
-    if (result && strlen(richTextHostName) > 63) {
+    mDNSBool result = mDNStrue;
+    if (result && strlen(richTextName) > 63) {
         if (printExplanation) {
             fprintf(stderr, 
-                    "%s: Host name is too long (must be 63 characters or less)\n", 
+                    "%s: Service name is too long (must be 63 characters or less)\n", 
                     gProgramName);
         }
         result = mDNSfalse;
     }
-    if (result && richTextHostName[0] == 0) {
+    if (result && richTextName[0] == 0) {
         if (printExplanation) {
-            fprintf(stderr, "%s: Host name can't be empty\n", gProgramName);
+            fprintf(stderr, "%s: Service name can't be empty\n", gProgramName);
         }
         result = mDNSfalse;
     }
-    if (result) {
-        MakeDomainLabelFromLiteralString(&richLabel, richTextHostName);
-        ConvertUTF8PstringToRFC1034HostLabel(richLabel.c, &poorLabel);
-        if (poorLabel.c[0] == 0) {
-            if (printExplanation) {
-                fprintf(stderr, 
-                        "%s: Host name doesn't produce a usable RFC-1034 name\n", 
-                        gProgramName);
-            }
-            result = mDNSfalse;
-        }
-    }
     return result;
 }
 
@@ -250,91 +266,6 @@ static mDNSBool CheckThatServiceTypeIsUsable(const char *serviceType, mDNSBool p
     return result;
 }
 
-static mDNSBool CheckThatServiceTextIsUsable(const char *serviceText, mDNSBool printExplanation,
-                                             mDNSu8 *pStringList, mDNSu16 *pStringListLen)
-    // Checks that serviceText is a reasonable service text record 
-    // and, if it isn't and printExplanation is true, prints 
-    // an explanation of why not.  Also parse the text into 
-    // the packed PString buffer denoted by pStringList and 
-    // return the length of that buffer in *pStringListLen.
-    // Note that this routine assumes that the buffer is 
-    // sizeof(RDataBody) bytes long.
-{
-    mDNSBool result;
-    size_t   serviceTextLen;
-    
-    // Note that parsing a C string into a PString list always 
-    // expands the data by one character, so the following 
-    // compare is ">=", not ">".  Here's the logic:
-    //
-    // #1 For a string with not ^A's, the PString length is one 
-    //    greater than the C string length because we add a length 
-    //    byte.
-    // #2 For every regular (not ^A) character you add to the C 
-    //    string, you add a regular character to the PString list.
-    //    This does not affect the equivalence stated in #1.
-    // #3 For every ^A you add to the C string, you add a length 
-    //    byte to the PString list but you also eliminate the ^A, 
-    //    which again does not affect the equivalence stated in #1.
-    
-    result = mDNStrue;
-    serviceTextLen = strlen(serviceText);
-    if (result && strlen(serviceText) >= sizeof(RDataBody)) {
-        if (printExplanation) {
-            fprintf(stderr, 
-                    "%s: Service text record is too long (must be less than %d characters)\n", 
-                    gProgramName,
-                    (int) sizeof(RDataBody) );
-        }
-        result = mDNSfalse;
-    }
-    
-    // Now break the string up into PStrings delimited by ^A.
-    // We know the data will fit so we can ignore buffer overrun concerns. 
-    // However, we still have to treat runs long than 255 characters as
-    // an error.
-    
-    if (result) {
-        int         lastPStringOffset;
-        int         i;
-        int         thisPStringLen;
-        
-        // This algorithm is a little tricky.  We start by copying 
-        // the string directly into the output buffer, shifted up by 
-        // one byte.  We then fill in the first byte with a ^A. 
-        // We then walk backwards through the buffer and, for each 
-        // ^A that we find, we replace it with the difference between 
-        // its offset and the offset of the last ^A that we found
-        // (ie lastPStringOffset).
-        
-        memcpy(&pStringList[1], serviceText, serviceTextLen);
-        pStringList[0] = 1;
-        lastPStringOffset = serviceTextLen + 1;
-        for (i = serviceTextLen; i >= 0; i--) {
-            if ( pStringList[i] == 1 ) {
-                thisPStringLen = (lastPStringOffset - i - 1);
-                assert(thisPStringLen >= 0);
-                if (thisPStringLen > 255) {
-                    result = mDNSfalse;
-                    if (printExplanation) {
-                        fprintf(stderr, 
-                                "%s: Each component of the service text record must be 255 characters or less\n", 
-                                gProgramName);
-                    }
-                    break;
-                } else {
-                    pStringList[i]    = thisPStringLen;
-                    lastPStringOffset = i;
-                }
-            }
-        }
-        
-        *pStringListLen = serviceTextLen + 1;
-    }
-
-    return result;
-}
-
 static mDNSBool CheckThatPortNumberIsUsable(long portNumber, mDNSBool printExplanation)
     // Checks that portNumber is a reasonable port number
     // and, if it isn't and printExplanation is true, prints 
@@ -368,7 +299,7 @@ enum {
 static void PrintUsage()
 {
     fprintf(stderr, 
-            "Usage: %s [-v level ] [-r] [-n name] [-t type] [-d domain] [-x TXT] [-p port] [-f file] [-b] [-P pidfile]\n", 
+            "Usage: %s [-v level ] [-r] [-n name] [-t type] [-d domain] [-p port] [-f file] [-b] [-P pidfile] [-x name=val ...]\n", 
             gProgramName);
     fprintf(stderr, "          -v verbose mode, level is a number from 0 to 2\n");
     fprintf(stderr, "             0 = no debugging info (default)\n");
@@ -376,20 +307,22 @@ static void PrintUsage()
     fprintf(stderr, "             2 = intense debugging info\n");
     fprintf(stderr, "             can be cycled kill -USR1\n");
     fprintf(stderr, "          -r also bind to port 53 (port 5353 is always bound)\n");
-    fprintf(stderr, "          -n uses 'name' as the host name (default is none)\n");
+    fprintf(stderr, "          -n uses 'name' as the service name (required)\n");
     fprintf(stderr, "          -t uses 'type' as the service type (default is '%s')\n", kDefaultServiceType);
     fprintf(stderr, "          -d uses 'domain' as the service domain (default is '%s')\n", kDefaultServiceDomain);
-    fprintf(stderr, "          -x uses 'TXT' as the service TXT record (default is empty)\n");
     fprintf(stderr, "          -p uses 'port' as the port number (default is '%d')\n",  kDefaultPortNumber);
     fprintf(stderr, "          -f reads a service list from 'file'\n");
     fprintf(stderr, "          -b forces daemon (background) mode\n");
     fprintf(stderr, "          -P uses 'pidfile' as the PID file\n");
     fprintf(stderr, "             (default is '%s')\n",  kDefaultPIDFile);
     fprintf(stderr, "             only meaningful if -b also specified\n");
+    fprintf(stderr, "          -x stores name=val in TXT record (default is empty).\n");
+    fprintf(stderr, "             MUST be the last command-line argument;\n");
+    fprintf(stderr, "             all subsequent arguments after -x are treated as name=val pairs.\n");
 }
 
 static   mDNSBool  gAvoidPort53      = mDNStrue;
-static const char *gRichTextHostName = "";
+static const char *gServiceName      = "";
 static const char *gServiceType      = kDefaultServiceType;
 static const char *gServiceDomain    = kDefaultServiceDomain;
 static mDNSu8      gServiceText[sizeof(RDataBody)];
@@ -417,7 +350,7 @@ static void ParseArguments(int argc, char **argv)
     // Parse command line options using getopt.
     
     do {
-        ch = getopt(argc, argv, "v:rn:t:d:x:p:f:dPb");
+        ch = getopt(argc, argv, "v:rn:t:d:p:f:dP:bx");
         if (ch != -1) {
             switch (ch) {
                 case 'v':
@@ -433,8 +366,8 @@ static void ParseArguments(int argc, char **argv)
                     gAvoidPort53 = mDNSfalse;
                     break;
                 case 'n':
-                    gRichTextHostName = optarg;
-                    if ( ! CheckThatRichTextHostNameIsUsable(gRichTextHostName, mDNStrue) ) {
+                    gServiceName = optarg;
+                    if ( ! CheckThatRichTextNameIsUsable(gServiceName, mDNStrue) ) {
                         exit(1);
                     }
                     break;
@@ -447,11 +380,6 @@ static void ParseArguments(int argc, char **argv)
                 case 'd':
                     gServiceDomain = optarg;
                     break;
-                case 'x':
-                    if ( ! CheckThatServiceTextIsUsable(optarg, mDNStrue, gServiceText, &gServiceTextLen) ) {
-                        exit(1);
-                    }
-                    break;
                 case 'p':
                     gPortNumber = atol(optarg);
                     if ( ! CheckThatPortNumberIsUsable(gPortNumber, mDNStrue) ) {
@@ -467,6 +395,16 @@ static void ParseArguments(int argc, char **argv)
                 case 'P':
                     gPIDFile = optarg;
                     break;
+                case 'x':
+                	while (optind < argc)
+                		{
+                		gServiceText[gServiceTextLen] = strlen(argv[optind]);
+                		memcpy(gServiceText+gServiceTextLen+1, argv[optind], gServiceText[gServiceTextLen]);
+                		gServiceTextLen += 1 + gServiceText[gServiceTextLen];
+                		optind++;
+                		}
+                	ch = -1;
+                	break;
                 case '?':
                 default:
                     PrintUsage();
@@ -486,9 +424,9 @@ static void ParseArguments(int argc, char **argv)
     
     // Check for inconsistency between the arguments.
     
-    if ( (gRichTextHostName[0] == 0) && (gServiceFile[0] == 0) ) {
+    if ( (gServiceName[0] == 0) && (gServiceFile[0] == 0) ) {
     	PrintUsage();
-        fprintf(stderr, "%s: You must specify a service to register (-n) or a service file (-f).\n", gProgramName);
+        fprintf(stderr, "%s: You must specify a service name to register (-n) or a service file (-f).\n", gProgramName);
         exit(1);
     }
 }
@@ -515,13 +453,13 @@ static void RegistrationCallback(mDNS *const m, ServiceRecordSet *const thisRegi
     switch (status) {
 
         case mStatus_NoError:      
-            debugf("Callback: %##s Name Registered",   thisRegistration->RR_SRV.resrec.name.c); 
+            debugf("Callback: %##s Name Registered",   thisRegistration->RR_SRV.resrec.name->c); 
             // Do nothing; our name was successfully registered.  We may 
             // get more call backs in the future.
             break;
 
         case mStatus_NameConflict: 
-            debugf("Callback: %##s Name Conflict",     thisRegistration->RR_SRV.resrec.name.c); 
+            debugf("Callback: %##s Name Conflict",     thisRegistration->RR_SRV.resrec.name->c); 
 
             // In the event of a conflict, this sample RegistrationCallback 
             // just calls mDNS_RenameAndReregisterService to automatically 
@@ -539,7 +477,7 @@ static void RegistrationCallback(mDNS *const m, ServiceRecordSet *const thisRegi
             break;
 
         case mStatus_MemFree:      
-            debugf("Callback: %##s Memory Free",       thisRegistration->RR_SRV.resrec.name.c); 
+            debugf("Callback: %##s Memory Free",       thisRegistration->RR_SRV.resrec.name->c); 
             
             // When debugging is enabled, make sure that thisRegistration 
             // is not on our gServiceList.
@@ -559,14 +497,14 @@ static void RegistrationCallback(mDNS *const m, ServiceRecordSet *const thisRegi
             break;
 
         default:                   
-            debugf("Callback: %##s Unknown Status %d", thisRegistration->RR_SRV.resrec.name.c, status); 
+            debugf("Callback: %##s Unknown Status %ld", thisRegistration->RR_SRV.resrec.name->c, status); 
             break;
     }
 }
 
 static int gServiceID = 0;
 
-static mStatus RegisterOneService(const char *  richTextHostName, 
+static mStatus RegisterOneService(const char *  richTextName, 
                                   const char *  serviceType, 
                                   const char *  serviceDomain, 
                                   const mDNSu8  text[],
@@ -585,7 +523,7 @@ static mStatus RegisterOneService(const char *  richTextHostName,
         status = mStatus_NoMemoryErr;
     }
     if (status == mStatus_NoError) {
-        MakeDomainLabelFromLiteralString(&name,  richTextHostName);
+        MakeDomainLabelFromLiteralString(&name,  richTextName);
         MakeDomainNameFromDNSNameString(&type, serviceType);
         MakeDomainNameFromDNSNameString(&domain, serviceDomain);
         status = mDNS_RegisterService(&mDNSStorage, &thisServ->coreServ,
@@ -608,7 +546,7 @@ static mStatus RegisterOneService(const char *  richTextHostName,
                     "%s: Registered service %d, name '%s', type '%s', port %ld\n", 
                     gProgramName, 
                     thisServ->serviceID, 
-                    richTextHostName,
+                    richTextName,
                     serviceType,
                     portNumber);
         }
@@ -621,91 +559,98 @@ static mStatus RegisterOneService(const char *  richTextHostName,
 }
 
 static mDNSBool ReadALine(char *buf, size_t bufSize, FILE *fp)
+// Read a line, skipping over any blank lines or lines starting with '#'
+{
+	mDNSBool good, skip;
+	do {
+		good = (fgets(buf, bufSize, fp) != NULL);
+		skip = (good && (buf[0] == '#'));
+	} while (good && skip);
+	if (good)
 	{
-    mDNSBool good = (fgets(buf, bufSize, fp) != NULL);
-    if (good)
-    	{
-        size_t len = strlen(buf);
-        good = (len > 0 && buf[len - 1] == '\n');
-		if (good) buf[len - 1] = 0;
-	    }
-    return good;
+		int		len = strlen( buf);
+		if ( buf[len - 1] == '\r' || buf[len - 1] == '\n')
+			buf[len - 1] = '\0';
 	}
+    return good;
+}
 
 static mStatus RegisterServicesInFile(const char *filePath)
 {
-    mStatus     status;
-    FILE *      fp;
+    mStatus     status = mStatus_NoError;
+    FILE *      fp = fopen(filePath, "r");
     int         junk;
-    mDNSBool    good;
-    int         ch;
-    char name[256];
-    char type[256];
-    const char *dom = kDefaultServiceDomain;
-    char rawText[1024];
-    mDNSu8  text[sizeof(RDataBody)];
-    mDNSu16 textLen;
-    char port[256];
     
-    status = mStatus_NoError;
-    fp = fopen(filePath, "r");
     if (fp == NULL) {
         status = mStatus_UnknownErr;
     }
     if (status == mStatus_NoError) {
-        good = mDNStrue;
+        mDNSBool good = mDNStrue;
         do {
+			int         ch;
+			char name[256];
+			char type[256];
+			const char *dom = kDefaultServiceDomain;
+			char rawText[1024];
+			mDNSu8  text[sizeof(RDataBody)];
+			unsigned int textLen = 0;
+			char port[256];
+
             // Skip over any blank lines.
-            do {
-                ch = fgetc(fp);
-            } while ( ch == '\n' || ch == '\r' );
-            if (ch != EOF) {
-                good = (ungetc(ch, fp) == ch);
-            }
+            do ch = fgetc(fp); while ( ch == '\n' || ch == '\r' );
+            if (ch != EOF) good = (ungetc(ch, fp) == ch);
             
             // Read three lines, check them for validity, and register the service.
-            if ( good && ! feof(fp) ) {
-                good = ReadALine(name, sizeof(name), fp);               
-                if (good) {
-                    good = ReadALine(type, sizeof(type), fp);
-                }
-                if (good) {
-                	char *p = type;
-                	while (*p && *p != ' ') p++;
-                	if (*p) {
-                		*p = 0;
-                		dom = p+1;
-                	}
-                }
-                if (good) {
-                    good = ReadALine(rawText, sizeof(rawText), fp);
-                }
-                if (good) {
-                    good = ReadALine(port, sizeof(port), fp);
-                }
-                if (good) {
-                    good =     CheckThatRichTextHostNameIsUsable(name, mDNSfalse)
-                            && CheckThatServiceTypeIsUsable(type, mDNSfalse)
-                            && CheckThatServiceTextIsUsable(rawText, mDNSfalse, text, &textLen)
-                            && CheckThatPortNumberIsUsable(atol(port), mDNSfalse);
-                }
-                if (good) {
-                    status = RegisterOneService(name, type, dom, text, textLen, atol(port));
-                    if (status != mStatus_NoError) {
-                        fprintf(stderr, 
-                                "%s: Failed to register service, name = %s, type = %s, port = %s\n", 
-                                gProgramName,
-                                name,
-                                type,
-                                port);
-                        status = mStatus_NoError;       // keep reading
-                    }
-                }
-            }
+			good = ReadALine(name, sizeof(name), fp);               
+			if (good) {
+				good = ReadALine(type, sizeof(type), fp);
+			}
+			if (good) {
+				char *p = type;
+				while (*p && *p != ' ') p++;
+				if (*p) {
+					*p = 0;
+					dom = p+1;
+				}
+			}
+			if (good) {
+				good = ReadALine(port, sizeof(port), fp);
+			}
+			if (good) {
+				good =     CheckThatRichTextNameIsUsable(name, mDNSfalse)
+						&& CheckThatServiceTypeIsUsable(type, mDNSfalse)
+						&& CheckThatPortNumberIsUsable(atol(port), mDNSfalse);
+			}
+			if (good) {
+				while (1) {
+					int len;
+					if (!ReadALine(rawText, sizeof(rawText), fp)) break;
+					len = strlen(rawText);
+					if (len <= 255)
+						{
+						unsigned int newlen = textLen + 1 + len;
+						if (len == 0 || newlen >= sizeof(text)) break;
+						text[textLen] = len;
+						memcpy(text + textLen + 1, rawText, len);
+						textLen = newlen;
+						}
+					else
+						fprintf(stderr, "%s: TXT attribute too long for name = %s, type = %s, port = %s\n", 
+							gProgramName, name, type, port);
+				}
+			}
+			if (good) {
+				status = RegisterOneService(name, type, dom, text, textLen, atol(port));
+				if (status != mStatus_NoError) {
+					fprintf(stderr, "%s: Failed to register service, name = %s, type = %s, port = %s\n", 
+							gProgramName, name, type, port);
+					status = mStatus_NoError;       // keep reading
+				}
+			}
         } while (good && !feof(fp));
 
         if ( ! good ) {
-            fprintf(stderr, "%s: Error reading service file %s\n", gProgramName, gServiceFile);
+            fprintf(stderr, "%s: Error reading service file %s\n", gProgramName, filePath);
         }
     }
     
@@ -722,8 +667,8 @@ static mStatus RegisterOurServices(void)
     mStatus status;
     
     status = mStatus_NoError;
-    if (gRichTextHostName[0] != 0) {
-        status = RegisterOneService(gRichTextHostName, 
+    if (gServiceName[0] != 0) {
+        status = RegisterOneService(gServiceName, 
                                     gServiceType, 
                                     gServiceDomain, 
                                     gServiceText, gServiceTextLen, 
@@ -761,43 +706,6 @@ static void DeregisterOurServices(void)
 #pragma mark **** Main
 #endif
 
-#ifdef NOT_HAVE_DAEMON
-
-    // The version of Solaris that I tested on didn't have the daemon 
-    // call.  This implementation was basically stolen from the 
-    // Mac OS X standard C library.
-    
-    static int daemon(int nochdir, int noclose)
-    {
-        int fd;
-    
-        switch (fork()) {
-        case -1:
-            return (-1);
-        case 0:
-            break;
-        default:
-            _exit(0);
-        }
-    
-        if (setsid() == -1)
-            return (-1);
-    
-        if (!nochdir)
-            (void)chdir("/");
-    
-        if (!noclose && (fd = _open("/dev/null", O_RDWR, 0)) != -1) {
-            (void)dup2(fd, STDIN_FILENO);
-            (void)dup2(fd, STDOUT_FILENO);
-            (void)dup2(fd, STDERR_FILENO);
-            if (fd > 2)
-                (void)_close(fd);
-        }
-        return (0);
-    }
-
-#endif /* NOT_HAVE_DAEMON */
-
 int main(int argc, char **argv)
 {
     mStatus status;
@@ -923,7 +831,7 @@ int main(int argc, char **argv)
         result = 2;
     }
     if ( (result != 0) || (gMDNSPlatformPosixVerboseLevel > 0) ) {
-        fprintf(stderr, "%s: Finished with status %ld, result %d\n", gProgramName, status, result);
+        fprintf(stderr, "%s: Finished with status %d, result %d\n", gProgramName, (int)status, result);
     }
     
     return result;
